using System;
using System.Collections.Generic;
using System.Text;
using SideShow.Events;
using System.Net;
using System.Collections.ObjectModel;

namespace SideShow.Channel
{
    public class ClientApplicationManager : IServerChannel, IClientChannel
    {
        private SideShowApplicationCollection _applications;
        private SideShowApplication _currentApplication;
        private PocketPCClientChannel _channel;
        private Guid _endpointId;

        public event EventHandler ListChanged;
        
        public ClientApplicationManager(Guid endpointId)
        {
            _channel = new PocketPCClientChannel(this);
            _endpointId = endpointId;
        }

        public void Connect(IPAddress host, int port)
        {
            _channel.Open(host, port);
        }

        public void Close()
        {
            _channel.Close();
        }

        public SideShowApplicationCollection Applications
        {
            get
            {
                if (_applications == null)
                {
                    _applications = new SideShowApplicationCollection();
                    _applications.ListChanged += new EventHandler(OnListChanged);
                }
                return _applications;
            }
        }

        private void OnListChanged(object sender, EventArgs e)
        {
            if (ListChanged != null)
            {
                ListChanged(this, e);
            }
        }

        public SideShowApplication CurrentApplication
        {
            get { return _currentApplication; }
            set 
            {
                if (_currentApplication != value)
                {
                    if (_currentApplication != null)
                    {
                        ApplicationExitEvent e = new ApplicationExitEvent(_currentApplication.Id, _endpointId);
                        _channel.PostEvent(e);
                    }
                    _currentApplication = value;
                    if (_currentApplication != null)
                    {
                        ApplicationEnterEvent e = new ApplicationEnterEvent(_currentApplication.Id, _endpointId);
                        _channel.PostEvent(e);
                    }

                }
            }
        }

        
        #region IPocketPCChannel Members

        void IServerChannel.AddApplication(SideShowApplication app)
        {
            Applications.Add(app);
        }

        void IServerChannel.AddContent(Guid applicationId, Guid sideShowEndpointId, SideShow.Contents.Content newContent)
        {
            Applications[applicationId].Endpoints[sideShowEndpointId].AddContent(newContent);
        }

        void IServerChannel.RemoveApplication(Guid appId)
        {
            Applications.Remove(appId);
        }

        void IServerChannel.RemoveContent(Guid applicationId, Guid sideShowEndpointId, uint removedContentId)
        {
            Applications[applicationId].Endpoints[sideShowEndpointId].RemoveContent(removedContentId);
        }

        void IServerChannel.Initialize()
        {
            
        }

        void IServerChannel.Deinitialize()
        {
            
        }

        #endregion

        #region ICallbackChannel Members

        void IClientChannel.PostEvent(SideShowEvent e)
        {
            _channel.PostEvent(e);
        }

        #endregion
    }
}
