using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using SideShow.Contents;
using SideShow.Scf.ObjectModel;
using System.Xml.Serialization;

namespace SideShow.Scf.Contents
{
    [Serializable]
    public class ScfContent : Content
    {
        [NonSerialized]
        private XmlDocument _rawXmlDoc;
        private ScfDocument _scfDocument;

        public ScfContent()
        {
        }

        public ScfContent(uint id, XmlDocument scfDoc) : base(id)
        {
            _rawXmlDoc = scfDoc;
            _scfDocument = ScfDocument.Load(_rawXmlDoc);
        }

        
        public ScfDocument ScfDocument
        {
            get
            {
                if (_scfDocument == null)
                {
                    _scfDocument = ScfDocument.Load(_rawXmlDoc);
                }
                return _scfDocument;
            }
        }

        public override void Save(string dir)
        {
            string filename = Path.Combine(dir, Id.ToString() + ".xml");
            _rawXmlDoc.Save(filename);
        }

        protected internal override void DeserializeObject(BinaryReader reader)
        {
            base.DeserializeObject(reader);
            _scfDocument = new ScfDocument();
            _scfDocument.DeserializeObject(reader);
        }

        protected internal override void SerializeObject(BinaryWriter writer)
        {
            base.SerializeObject(writer);
            ScfDocument.SerializeObject(writer);
        }
    }
}
